<?php

require_once 'loginsecurity.civix.php';
// phpcs:disable
use CRM_Loginsecurity_ExtensionUtil as E;
// phpcs:enable

$autoload = __DIR__ . '/vendor/autoload.php';
if (file_exists($autoload) && version_compare(phpversion(), '8.1.0', '>=')) {
  require_once $autoload;
}

/**
 * Implements hook_civicrm_config().
 *
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_config/
 */
function loginsecurity_civicrm_config(&$config) {
  _loginsecurity_civix_civicrm_config($config);

  if (isset(Civi::$statics[__FUNCTION__])) { return; }
  Civi::$statics[__FUNCTION__] = 1;

  // Run with a high priority, to run before other export hooks which may halt execution
  Civi::dispatcher()->addListener('hook_civicrm_export', '_loginsecurity_civicrm_export', 500);

  $result = CRM_Loginsecurity_BAO_LoginSecurityDevice::logCurrentSession();

  if (!empty($result['contact_id']) && empty($result['fingerprint']) && empty(CRM_Utils_Request::retrieveValue('snippet', 'String'))) {
    Civi::resources()
      ->addVars('loginsecurity', $result)
      ->addScriptFile('loginsecurity', 'node_modules/@fingerprintjs/fingerprintjs/dist/fp.min.js')
      ->addScriptFile('loginsecurity', 'loginsecurity.js');
  }

  // If an OTP was set, then require the OTP
  $path = CRM_Utils_System::currentPath();
  if ($path != 'civicrm/loginsecurity/otp') {
    try {
      $otp = Civi::contactSettings()->get('loginsecurity_otp');
      if ($otp) {
        $contact_id = CRM_Core_Session::getLoggedInContactID();
        $session_id = session_id();
        $device = \Civi\Api4\LoginSecurityDevice::get(FALSE)
          ->addWhere('contact_id', '=', $contact_id)
          ->addWhere('session_id', '=', $session_id)
          ->addWhere('is_trusted', '=', 1)
          ->execute()
          ->first();
        if (empty($device['is_trusted'])) {
          CRM_Utils_System::redirect(CRM_Utils_System::url('civicrm/loginsecurity/otp', 'reset=1'));
        }
      }
    }
    catch (Exception $e) {
      // The settings subsystem is not available during a cache flush
    }
  }
}

/**
 * Implements hook_civicrm_apiWrappers().
 */
function loginsecurity_civicrm_apiWrappers(&$wrappers, $apiRequest) {
  // Ignoring API4 for now, because when we load the dashboard (or when an admin logins?)
  // there are some api4 calls that seem to run before the hook_civicrm_config has run,
  // resulting in a fatal CRM_Loginsecurity_APIWrappers class not found.
  // Also, there is no REST api4 endpoint at the moment (as of 5.42).
  // We could also probably limit the wrapper to run only for the more sensitive entities,
  // such as Contact/Activity/Contribution/Participant.
  if ($apiRequest['version'] == '3') {
    $wrappers[] = new CRM_Loginsecurity_APIWrappers();
  }
}

/**
 * Implements hook_civicrm_install().
 *
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_install
 */
function loginsecurity_civicrm_install() {
  _loginsecurity_civix_civicrm_install();
}

/**
 * Implements hook_civicrm_enable().
 *
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_enable
 */
function loginsecurity_civicrm_enable() {
  _loginsecurity_civix_civicrm_enable();
}

/**
 * Implements hook_civicrm_navigationMenu().
 *
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_navigationMenu
 */
function loginsecurity_civicrm_navigationMenu(&$menu) {
  _loginsecurity_civix_insert_navigation_menu($menu, 'Administer/System Settings', [
    'label' => E::ts('Login Security'),
    'name' => 'loginsecurity_settings',
    'url' => 'civicrm/admin/setting/loginsecurity',
    'permission' => 'administer CiviCRM system',
    'operator' => 'OR',
    'separator' => 0,
  ]);
  _loginsecurity_civix_navigationMenu($menu);
}

/**
 * Implements hook_civicrm_tabset().
 */
function loginsecurity_civicrm_tabset($tabsetName, &$tabs, $context) {
  if ($tabsetName != 'civicrm/contact/view') {
    return;
  }

  $contact_id = $context['contact_id'];

  if (!$contact_id) {
    return;
  }

  // Users may view their own history, otherwise must be a system admin
  $is_self = CRM_Core_Session::getLoggedInContactID() == $contact_id;

  if (!(CRM_Core_Permission::check('administer CiviCRM system') || $is_self)) {
    return;
  }

  // Always display the tab if we are logging Petition Signatures
  if (!Civi::settings()->get('loginsecurity_log_petitions')) {
    // Check if the contact record is linked to a CMS user
    $uf_id = CRM_Core_DAO::singleValueQuery('SELECT uf_id from civicrm_uf_match WHERE contact_id = %1', [
      1 => [$contact_id, 'Positive'],
    ]);

    if (!$uf_id) {
      return;
    }
  }

  $url = CRM_Utils_System::url('civicrm/contact/loginhistory', "reset=1&snippet=1&force=1&cid=$contact_id");

  $tabs[] = [
    'id' => 'loginsecurityTab',
    'url'   => $url,
    'title' => E::ts('Access History'),
    'count' => '',
    'icon' => 'crm-i fa-lock',
    'contact_type' => ['Individual'],
    'weight' => 900,
  ];
}

/**
 * Implements hook_civicrm_check().
 */
function loginsecurity_civicrm_check(&$messages) {
  $checks = [
    'InvalidLogins',
  ];

  foreach ($checks as $c) {
    $t = 'CRM_Loginsecurity_Utils_Check_' . $c;
    if (class_exists($t)) {
      $class = new $t();
      $class->check($messages);
    }
  }
}

/**
 * Implements hook_civicrm_postCommit().
 *
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_postCommit
 */
function loginsecurity_civicrm_postCommit($op, $objectName, $objectId, &$objectRef) {
  // For new Petition signatures, log the access and also save the IP in the activity location field
  if (!Civi::settings()->get('loginsecurity_log_petitions')) {
    return;
  }

  if ($objectName == 'Activity' && $op == 'create') {
    $optionValue = \Civi\Api4\OptionValue::get(FALSE)
      ->addSelect('value')
      ->addWhere('option_group_id:name', '=', 'activity_type')
      ->addWhere('name', '=', 'Petition')
      ->execute()
      ->first();

    if ($optionValue && $optionValue['value'] == $objectRef->activity_type_id) {
      // Find the contact_id from the activity
      $contact_id = CRM_Core_DAO::singleValueQuery('SELECT contact_id FROM civicrm_activity_contact WHERE record_type_id = 3 AND activity_id = %1', [
        1 => [$objectId, 'Positive'],
      ]);

      $result = CRM_Loginsecurity_BAO_LoginSecurityDevice::logCurrentSession(['contact_id' => $contact_id]);

      if (!empty($result['contact_id'])) {
        \Civi\Api4\Activity::update(FALSE)
          ->addValue('location', $result['id'])
          ->addWhere('id', '=', $objectId)
          ->execute();

        if (empty($result['skip_fingerprint'])) {
          $session = CRM_Core_Session::singleton();
          $session->set('loginsecurity_device', $result);
        }
      }
    }
  }
}

/**
 * Implements hook_civicrm_esmImportMap()
 */
function loginsecurity_civicrm_esmImportMap(\Civi\Esm\ImportMap $importMap): void {
  // @todo FIXME obscure JS error if we include the minified version
  // $importMap->addPrefix('@simplewebauthn/browser', E::LONG_NAME, 'node_modules/@simplewebauthn/browser/dist/bundle/index.umd.min.js');
  $importMap->addPrefix('simplewebauthn/browser', E::LONG_NAME, 'node_modules/@simplewebauthn/browser/dist/bundle/index.js');
}

/**
 * Implements hook_civicrm_pageRun()
 */
function loginsecurity_civicrm_pageRun(&$page) {
  $pageName = get_class($page);

  if ($pageName == 'CRM_Contact_Page_View_Summary') {
    // Load JS for webauthn on Login Security tab
    Civi::resources()->addModuleFile(E::LONG_NAME, 'js/webauthn.js');
  }
  elseif ($pageName == 'CRM_Campaign_Page_Petition_ThankYou') {
    if (!Civi::settings()->get('loginsecurity_log_petitions')) {
      return;
    }

    $session = CRM_Core_Session::singleton();
    $device = $session->get('loginsecurity_device');

    if (!empty($device) && !empty($device['id'])) {
      Civi::resources()
        ->addVars('loginsecurity', $device)
        ->addScriptFile('loginsecurity', 'node_modules/@fingerprintjs/fingerprintjs/dist/fp.min.js')
        ->addScriptFile('loginsecurity', 'loginsecurity.js');
    }
  }
}

/**
 * Implements hook_civicrm_export() using Symfony hooks.
 *
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_export/
 */
function _loginsecurity_civicrm_export($event) {
  $exportTypes = [
    1 => 'Contact',
    2 => 'Contribute',
    3 => 'Membership',
    4 => 'Event',
    5 => 'Pledge',
    6 => 'Case',
    7 => 'Grant',
    8 => 'Activity',
  ];

  $printExportMode = $exportTypes[$event->exportMode] ?? 'Unknown';
  $printHeaderRows = json_encode($event->headerRows);
  $printIds = json_encode($event->ids);
  $row_count = count($event->ids);
  $contact_id = CRM_Core_Session::getLoggedInContactID();
  $ip_address = CRM_Loginsecurity_Utils_System::ipAddress();

  \Civi::log()->info("Export: Contact {$contact_id} exported {$row_count} records of type {$printExportMode} with the following Header Rows {$printHeaderRows} involving the following IDs {$printIds}");

  $admin_emails = Civi::settings()->get('loginsecurity_notify_export');

  if ($admin_emails) {
    $admin_emails = trim($admin_emails);
    $emails = explode(',', $admin_emails);

    foreach ($emails as $email) {
      $email = trim($email);

      CRM_Loginsecurity_Utils_Mail::sendNotification([
        'admin_email' => $email,
        'contact_id' => $contact_id,
        'ip_address' => $ip_address,
        'loginsecurity_email_subject' => E::ts('CiviCRM Data Export: %1 %2 records', [1 => $row_count, 2 => $printExportMode]),
        'export_mode' => $printExportMode,
        'export_headers' => $printHeaderRows,
        'export_row_count' => $row_count,
        'valueName' => 'loginsecurity_export_notification',
      ]);
    }
  }
}

function smarty_modifier_loginsecurity_location($id) {
  $ip_address = smarty_modifier_loginsecurity_ip($id);

  if (!$ip_address || $ip_address == 'error') {
    return 'error';
  }

  return CRM_Loginsecurity_BAO_LoginSecurityDevice::getGeoIP($ip_address);
}

function smarty_modifier_loginsecurity_ip($id) {
  $result = \Civi\Api4\LoginSecurityDevice::get(FALSE)
    ->addWhere('id', '=', $id)
    ->execute()
    ->first();

  if (!$result) {
    return 'error';
  }

  return $result['ip_address'];
}
